﻿using Buddy.Coroutines;
using ff14bot;
using ff14bot.AClasses;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Objects;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using TreeSharp;

namespace ChocoboFeeder
{
    public enum ChocoboFood : uint
    {
        Curiel_Root = 7894,
        Sylkis_Bud = 7895,
        Mimett_Gourd = 7897,
        Tantalplant = 7898,
        Pahsana_Fruit = 7900,

        Any
    }


    public class ChocoboFeederSettings : JsonSettings
    {

        private static ChocoboFeederSettings _settings;

        public static ChocoboFeederSettings Instance
        {
            get { return _settings ?? (_settings = new ChocoboFeederSettings()); }
        }

        public ChocoboFeederSettings()
            : base(Path.Combine(CharacterSettingsDirectory, "ChocoboFeederSettings.json"))
        {
        }

        private ChocoboFood _ChocoboFood;
        [Setting]
#if RB_CN
	[DisplayName("陆行鸟食物")]
	[Description("需要为陆行鸟喂食什么食物?")]
#else
        [DisplayName("ChocoboFood")]
        [Description("What food to fed the chocobos")]
#endif
        [DefaultValueAttribute(ChocoboFood.Curiel_Root)]
        public ChocoboFood ChocoboFood
        {
            get { return _ChocoboFood; }
            set
            {
                if (_ChocoboFood != value)
                {
                    _ChocoboFood = value;
                    Save();
                }
            }
        }
    }

    class ChocoboFeeder : BotPlugin
    {
        private TreeSharp.Composite Root;
        public override string Author
        {
            get
            {
                return "Eniac";
            }
        }

        public override string Name
        {
            get
            {
                return "ChocoboFeeder";
            }
        }

        public static readonly Version version = new Version(0, 1, 1);
        public override Version Version
        {
            get
            {
                return version;
            }
        }

        public override string ButtonText
        {
            get
            {
                return "Settings";
            }
        }

        public override bool WantButton
        {
            get
            {
                return true;
            }
        }

        private SettingsForm settings;

        public override void OnButtonPress()
        {
            if (settings == null || settings.IsDisposed)
                settings = new SettingsForm();
            try
            {
                settings.Show();
                settings.Activate();
            }
            catch (ArgumentOutOfRangeException ee)
            {

            }
        }

        override
        public void OnInitialize()
        {
        }

        override
        public void OnEnabled()
        {
            Root = new ActionRunCoroutine(r => FeedingLogic());
            Root.Start(null);
        }

        override
        public void OnDisabled()
        {
            Root.Stop(null);
            Root = null;
        }

        override
        public void OnShutdown()
        {
        }

        override
        public void OnPulse()
        {
            try
            {
                Root.Tick(null);
                if (Root.LastStatus != TreeSharp.RunStatus.Running)
                {
                    Root.Stop(null);
                    Root.Start(null);
                }
            }
            catch (Exception e)
            {
                ff14bot.Helpers.Logging.WriteException(e);
                Root.Stop(null);
                Root.Start(null);
                throw;
            }
        }

        public BagSlot Food(uint ItemId)
        {
            return InventoryManager.FilledSlots.FirstOrDefault(r => r.RawItemId == ItemId);
        }

        private async Task FeedingLogic()
        {
            if (Chocobo.Object != null && Chocobo.Object.IsVisible && Chocobo.Summoned && !Chocobo.Object.HasAura(536))
            {
                BagSlot currentFood = Food((uint)ChocoboFeederSettings.Instance.ChocoboFood);
                if (currentFood != null)
                {
                    currentFood.UseItem();
                    await Coroutine.Wait(300, () => Chocobo.Object.HasAura(536));
                }
            }
        }
    }
}

